@@ -5,6 +5,8 @@
  * Copyright (c) 2014-2016 Cisco Systems, Inc.  All Rights Reserved.
  */
 
+#define OPENSSL_API_COMPAT 0x10000000L
+
 #include <cjose/base64.h>
 #include <cjose/header.h>
 #include <cjose/jws.h>
@@ -51,7 +53,7 @@ static bool _cjose_jws_build_hdr(cjose_jws_t *jws, cjose_header_t *header, cjose
     json_incref(jws->hdr);
 
     // base64url encode the header
-    char *hdr_str = json_dumps(jws->hdr, JSON_ENCODE_ANY | JSON_PRESERVE_ORDER);
+    char *hdr_str = json_dumps(jws->hdr, JSON_ENCODE_ANY | JSON_PRESERVE_ORDER | JSON_COMPACT);
     if (NULL == hdr_str)
     {
         CJOSE_ERROR(err, CJOSE_ERR_NO_MEMORY);
@@ -59,10 +61,10 @@ static bool _cjose_jws_build_hdr(cjose_jws_t *jws, cjose_header_t *header, cjose
     }
     if (!cjose_base64url_encode((const uint8_t *)hdr_str, strlen(hdr_str), &jws->hdr_b64u, &jws->hdr_b64u_len, err))
     {
-        free(hdr_str);
+        cjose_get_dealloc()(hdr_str);
         return false;
     }
-    free(hdr_str);
+    cjose_get_dealloc()(hdr_str);
 
     return true;
 }
@@ -171,6 +173,12 @@ static bool _cjose_jws_build_dig_sha(cjose_jws_t *jws, const cjose_jwk_t *jwk, c
         goto _cjose_jws_build_dig_sha_cleanup;
     }
 
+    if (NULL != jws->dig)
+    {
+        cjose_get_dealloc()(jws->dig);
+        jws->dig = NULL;
+    }
+
     // allocate buffer for digest
     jws->dig_len = EVP_MD_size(digest_alg);
     jws->dig = (uint8_t *)cjose_get_alloc()(jws->dig_len);
@@ -258,6 +266,12 @@ static bool _cjose_jws_build_dig_hmac_sha(cjose_jws_t *jws, const cjose_jwk_t *j
         goto _cjose_jws_build_dig_hmac_sha_cleanup;
     }
 
+    if (NULL != jws->dig)
+    {
+        cjose_get_dealloc()(jws->dig);
+        jws->dig = NULL;
+    }
+
     // allocate buffer for digest
     jws->dig_len = EVP_MD_size(digest_alg);
     jws->dig = (uint8_t *)cjose_get_alloc()(jws->dig_len);
